/* Call BLAS SGEMM to make a matrix multiplication */
/* Fernando G. Tinetti                             */

#include<stdio.h>
#include<stdlib.h>   /* malloc(), rand() */

#include "cblas.h"   /* C interface _gemm definition */

#define ELTYPE     double
#define GEMM_FCTN  cblas_dgemm


/* Init square matrix with a specific value */
void initvalmat(ELTYPE *mat, int n, ELTYPE val); 

/* Time in seconds from some point in the past */
double dwalltime();



/************** MAIN *************/
main(int argc, char *argv[])
{
  double *a, *b, *c;
  int m, n, k, i, j;

  double timetick;

  /* Check command line parameters */
  if ((argc != 2) || ((m = atoi(argv[1])) <= 0) )
  {
    printf("\nUsage: %s n\n  n: matrix order (nxn X nxn)\n", argv[0]);
    exit(1);
  }

  /* Just because n and k are not "command line" parameters */
  n = m;
  k = m;

  /* Getting memory */
  a = (ELTYPE *) malloc(m*k * sizeof(ELTYPE));
  b = (ELTYPE *) malloc(k*n * sizeof(ELTYPE));
  c = (ELTYPE *) malloc(m*n * sizeof(ELTYPE));

  /* Init matrix operands */
  initvalmat(a, n, 1.0);
  initvalmat(b, n, 1.0);

  printf("Multiplying %d x %d matrices\n", m, m);

  timetick = dwalltime();

  GEMM_FCTN(CblasRowMajor, CblasNoTrans, CblasNoTrans, m, n, k, 
              1.0, a, k, b, n, 0.0, c, n);

  printf("Back from multiply in %f seconds\n", dwalltime() - timetick);

  /* Check results (just in case...) */
  for (i = 0; i < n; i++)
  {
    for (j = 0; j < n; j++)
    {
      if (c[i*n + j] != n)
      {
        printf("Error at %i, %j, value: %f\n", i, j, c[i*n + j]);
      }
    }
  }
}

/*****************************************************************/

/* Init square matrix with a specific value */
void initvalmat(ELTYPE *mat, int n, ELTYPE val)
{
  int i, j;      /* Indexes */

  for (i = 0; i < n; i++)
  {
    for (j = 0; j < n; j++)
    {
      mat[i*n + j] = val;
    }
  }
}

/*****************************************************************/

#include <stdio.h>
#include <sys/time.h>

double dwalltime()
{
	double sec;
	struct timeval tv;

	gettimeofday(&tv,NULL);
	sec = tv.tv_sec + tv.tv_usec/1000000.0;
	return sec;
}

/*****************************************************************/
